@@ -382,10 +382,6 @@ typedef enum {
 
 struct pci_device;
 
-typedef struct {
-    unsigned int dummy;
-} IsaBusId;
-
 typedef struct {
     int		fbNum;
 } SbusBusId;
@@ -393,7 +389,6 @@ typedef struct {
 typedef struct _bus {
     BusType type;
     union {
-	IsaBusId isa;
 	struct pci_device *pci;
 	SbusBusId sbus;
     } id;
@@ -436,8 +431,6 @@ typedef struct {
    int                          screen;         /* For multi-CRTC cards */
 } GDevRec, *GDevPtr;
 
-typedef int (*FindIsaDevProc)(GDevPtr dev);
-
 typedef struct {
    char *			identifier;
    char *			driver;
@@ -623,9 +616,6 @@ typedef struct _CurrAccRec {
 
 #define ResMem		0x0001
 #define ResIo		0x0002
-#define ResIrq		0x0003
-#define ResDma		0x0004
-#define ResPciCfg	0x000e	/* PCI Configuration space */
 #define ResPhysMask	0x000F
 
 #define ResExclusive	0x0010
@@ -648,7 +638,6 @@ typedef struct _CurrAccRec {
 #define ResMiscMask	0x00F000
 
 #define ResBus		0x010000
-#define ResOverlap	0x020000
 
 #if defined(__alpha__) && defined(linux)
 # define ResDomain	0x1ff000000ul
@@ -687,7 +676,6 @@ typedef struct _CurrAccRec {
 #define ResIsBlock(r)		(((r)->type & ResExtMask) == ResBlock)
 #define ResIsSparse(r)		(((r)->type & ResExtMask) == ResSparse)
 #define ResIsEstimated(r)	(((r)->type & ResMiscMask) == ResEstimated)
-#define ResCanOverlap(r)	(ResIsEstimated(r) || ((r)->type & ResOverlap))
 
 typedef struct {
     unsigned long type;     /* shared, exclusive, unused etc. */
@@ -722,11 +710,6 @@ typedef struct _resRec {
 #define block_end	val.rEnd
 #define res_type	val.type
 
-typedef struct {
-    int numChipset;
-    resRange *resList;
-} IsaChipsets;
-
 typedef struct _PciChipsets {
     /**
      * Key used to match this device with its name in an array of
@@ -851,7 +834,6 @@ typedef int  xf86SetDGAModeProc           (int, int, DGADevicePtr);
 typedef int  xf86ChangeGammaProc          (int, Gamma);
 typedef void xf86PointerMovedProc         (int, int, int);
 typedef Bool xf86PMEventProc              (int, pmEvent, Bool);
-typedef int  xf86HandleMessageProc     (int, const char*, const char*, char**);
 typedef void xf86DPMSSetProc		  (ScrnInfoPtr, int, int);
 typedef void xf86LoadPaletteProc   (ScrnInfoPtr, int, int *, LOCO *, VisualPtr);
 typedef void xf86SetOverscanProc          (ScrnInfoPtr, int);
@@ -1006,7 +988,6 @@ typedef struct _ScrnInfoRec {
     xf86ChangeGammaProc			*ChangeGamma;
     xf86PointerMovedProc		*PointerMoved;
     xf86PMEventProc			*PMEvent;
-    xf86HandleMessageProc		*HandleMessage;
     xf86DPMSSetProc			*DPMSSet;
     xf86LoadPaletteProc			*LoadPalette;
     xf86SetOverscanProc			*SetOverscan;
@@ -1090,14 +1071,6 @@ typedef void (*InputHandlerProc)(int fd, pointer data);
 #define CLK_REG_SAVE		-1
 #define CLK_REG_RESTORE		-2
 
-/* xf86Debug.c */
-#ifdef BUILDDEBUG
-typedef struct {
-    long sec;
-    long usec;
-} xf86TsRec, *xf86TsPtr;
-#endif
-
 /*
  * misc constants
  */
@@ -1111,13 +1084,6 @@ typedef struct {
 #define OVERLAY_8_16_DUALFB	0x00000004
 #define OVERLAY_8_32_PLANAR	0x00000008
 
-#if 0
-#define LD_RESOLV_IFDONE		0	/* only check if no more
-						   delays pending */
-#define LD_RESOLV_NOW			1	/* finish one delay step */
-#define LD_RESOLV_FORCE			2	/* force checking... */
-#endif
-
 /* Values of xf86Info.mouseFlags */
 #define MF_CLEAR_DTR       1
 #define MF_CLEAR_RTS       2
@@ -1127,36 +1093,9 @@ typedef enum {
     ACTION_TERMINATE		= 0,	/* Terminate Server */
     ACTION_NEXT_MODE		= 10,	/* Switch to next video mode */
     ACTION_PREV_MODE,
-    ACTION_DISABLEGRAB		= 20,	/* Cancel server/pointer/kbd grabs */
-    ACTION_CLOSECLIENT,			/* Kill client holding grab */
     ACTION_SWITCHSCREEN		= 100,	/* VT switch */
     ACTION_SWITCHSCREEN_NEXT,
     ACTION_SWITCHSCREEN_PREV,
-    ACTION_MESSAGE		= 9999  /* Generic message passing */
 } ActionEvent;
 
-/* xf86Versions.c */
-/*
- * Never change existing values, and always assign values explicitly.
- * NUM_BUILTIN_IFS must always be the last entry.
- */
-typedef enum {
-    BUILTIN_IF_OSMOUSE = 0,
-    BUILTIN_IF_OSKBD = 1,
-    NUM_BUILTIN_IFS
-} BuiltinInterface;
-
-/*
- * These are intentionally the same as the module version macros.
- * It is possible to register a module as providing a specific interface,
- * in which case the module's version is used.  This feature isn't
- * really ready for use yet though.
- */
-
-#define BUILTIN_INTERFACE_VERSION_NUMERIC(maj, min, patch) \
-	((((maj) & 0xFF) << 24) | (((min) & 0xFF) << 16) | (patch & 0xFFFF))
-#define GET_BUILTIN_INTERFACE_MAJOR_VERSION(vers)	(((vers) >> 24) & 0xFF)
-#define GET_BUILTIN_INTERFACE_MINOR_VERSION(vers)	(((vers) >> 16) & 0xFF)
-#define GET_BUILTIN_INTERFACE_PATCH_VERSION(vers)	((vers) & 0xFFFF)
-
 #endif /* _XF86STR_H */
