@@ -39,7 +39,8 @@ is" without express or implied warranty.
 #include "Events.h"
 #include "Args.h"
 
-int xnestWindowPrivateIndex;
+static int xnestWindowPrivateKeyIndex;
+DevPrivateKey xnestWindowPrivateKey = &xnestWindowPrivateKeyIndex;
 
 static int
 xnestFindWindowMatch(WindowPtr pWin, pointer ptr)
@@ -131,12 +132,10 @@ xnestCreateWindow(WindowPtr pWin)
   xnestWindowPriv(pWin)->sibling_above = None;
   if (pWin->nextSib)
     xnestWindowPriv(pWin->nextSib)->sibling_above = xnestWindow(pWin);
-#ifdef SHAPE
   xnestWindowPriv(pWin)->bounding_shape = 
     REGION_CREATE(pWin->drawable.pScreen, NULL, 1);
   xnestWindowPriv(pWin)->clip_shape = 
     REGION_CREATE(pWin->drawable.pScreen, NULL, 1);
-#endif /* SHAPE */
 
   if (!pWin->parent) /* only the root window will have the right colormap */
     xnestSetInstalledColormapWindows(pWin->drawable.pScreen);
@@ -150,12 +149,10 @@ xnestDestroyWindow(WindowPtr pWin)
   if (pWin->nextSib)
     xnestWindowPriv(pWin->nextSib)->sibling_above = 
       xnestWindowPriv(pWin)->sibling_above;
-#ifdef SHAPE
   REGION_DESTROY(pWin->drawable.pScreen, 
 				xnestWindowPriv(pWin)->bounding_shape);
   REGION_DESTROY(pWin->drawable.pScreen, 
 				xnestWindowPriv(pWin)->clip_shape);
-#endif
   XDestroyWindow(xnestDisplay, xnestWindow(pWin));
   xnestWindowPriv(pWin)->window = None;
 
@@ -362,9 +359,7 @@ Bool
 xnestRealizeWindow(WindowPtr pWin)
 {
   xnestConfigureWindow(pWin, CWStackingOrder);
-#ifdef SHAPE
   xnestShapeWindow(pWin);
-#endif /* SHAPE */
   XMapWindow(xnestDisplay, xnestWindow(pWin));
 
   return True;
@@ -378,30 +373,6 @@ xnestUnrealizeWindow(WindowPtr pWin)
   return True;
 }
 
-void
-xnestPaintWindowBackground(WindowPtr pWin, RegionPtr pRegion, int what)
-{
-  int i;
-  BoxPtr pBox;
-
-  xnestConfigureWindow(pWin, CWWidth | CWHeight);
-
-  pBox = REGION_RECTS(pRegion);
-  for (i = 0; i < REGION_NUM_RECTS(pRegion); i++)
-    XClearArea(xnestDisplay, xnestWindow(pWin),
-	       pBox[i].x1 - pWin->drawable.x,
-	       pBox[i].y1 - pWin->drawable.y,
-	       pBox[i].x2 - pBox[i].x1, 
-	       pBox[i].y2 - pBox[i].y1,
-	       False);
-}
-
-void
-xnestPaintWindowBorder(WindowPtr pWin, RegionPtr pRegion, int what)
-{
-  xnestConfigureWindow(pWin, CWBorderWidth);
-}
-
 void
 xnestCopyWindow(WindowPtr pWin, xPoint oldOrigin, RegionPtr oldRegion)
 {
@@ -411,9 +382,7 @@ void
 xnestClipNotify(WindowPtr pWin, int dx, int dy)
 {
   xnestConfigureWindow(pWin, CWStackingOrder); 
-#ifdef SHAPE
   xnestShapeWindow(pWin);
-#endif /* SHAPE */
 }
 
 static Bool
@@ -450,7 +419,6 @@ xnestWindowExposures(WindowPtr pWin, RegionPtr pRgn, RegionPtr other_exposed)
   miWindowExposures(pWin, pRgn, other_exposed);
 }
 
-#ifdef SHAPE
 void
 xnestSetShape(WindowPtr pWin)
 {
@@ -553,4 +521,3 @@ xnestShapeWindow(WindowPtr pWin)
     }
   }
 }
-#endif /* SHAPE */
